package oiofs

import (
	"context"
	"fmt"
	"io/ioutil"
	"net/http"
	"net/http/httptest"
	"strconv"
	"strings"
	"testing"

	"github.com/open-io/oio-exporter/common"
	_log "github.com/open-io/oio-exporter/common/log"
	"github.com/open-io/oio-exporter/testutils"
)

func makeFakeOiofs(t *testing.T) (*httptest.Server, string, int) {
	data, err := ioutil.ReadFile("./oiofs.spec.json")
	if err != nil {
		t.Fatal("Couldn't read file 'oiofs.spec.json'")
	}

	server := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		if r.URL.Path != "/stats" {
			return
		}

		w.WriteHeader(200)
		if _, err := w.Write(data); err != nil {
			t.Fatalf("Failed to write 'oiofs.spec.json' content to response: %s", err)
		}
	}))

	parts := strings.Split(server.URL, ":")
	ip := parts[1][2:]
	port, err := strconv.ParseInt(parts[2], 10, 64)
	if err != nil {
		log.Fatalln(err)
	}

	return server, ip, int(port)
}

func TestOiofsModule(t *testing.T) {
	t.Parallel()
	_log.LogLevel = _log.INFO
	_ = _log.Setup()

	server, ip, port := makeFakeOiofs(t)
	defer server.Close()

	m := common.MakeMetricData()
	params := &oiofsDaemonParams{
		Namespace:     "OPENIO",
		ServiceID:     "fake-oiofs-0",
		ServiceIP:     ip,
		ServicePort:   port,
		ServiceVolume: "/fake/volume",
	}
	d := Factory.New(params)
	ctx := context.TODO()

	if err := d.Init(ctx); err != nil {
		t.Fatalf("Init error: %s", err)
	}

	if err := d.Collect(ctx, m); err != nil {
		t.Fatalf("Collect error: %s", err)
	}

	if err := d.Close(); err != nil {
		t.Fatalf("Close error: %s", err)
	}

	testutils.NewMetricDataTest().
		ForAllMetrics(
			testutils.NewMetricAssertion().
				LabelIs("service_type", "oiofs").
				LabelIs("service", "fake-oiofs-0").
				LabelIs("service_addr", fmt.Sprintf("%s:%d", ip, port)).
				LabelIs("mountpoint", "/fake/volume")).
		HasMetrics(
			"oiofs_operation_time",
			"oiofs_operation_count",
			"oiofs_bytes",
			"oiofs_chunk_age",
			"oiofs_cache_read",
		).
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(1),
			"oiofs_operation_count",
			"module", "meta",
			"operation", "initCtx",
			"dimension", "count").
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(158),
			"oiofs_operation_time",
			"module", "meta",
			"operation", "initCtx",
			"dimension", "avg").
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(17374182400),
			"oiofs_bytes",
			"module", "cache",
			"operation", "chunk",
			"dimension", "total").
		ForMetricWithLabels(
			testutils.NewMetricAssertion().Equals(123456),
			"oiofs_chunk_age",
			"module", "cache",
			"operation", "chunk",
			"dimension", "avg").
		NoExtraMetrics().
		Check(t, m)
}
